@@ -51,6 +51,11 @@ struct _YelpInfoPagerPriv {
     GHashTable    *frags_hash;
 };
 
+typedef struct {
+    gchar *frag_id;
+    gchar *frag_name;
+} hash_lookup;
+
 static void           info_pager_class_init   (YelpInfoPagerClass *klass);
 static void           info_pager_init         (YelpInfoPager      *pager);
 static void           info_pager_dispose      (GObject           *gobject);
@@ -66,6 +71,10 @@ static gboolean       tree_hash_id            (GtkTreeModel     *model,
 					       GtkTreePath      *path,
 					       GtkTreeIter      *iter,
 					       YelpInfoPager    *pager);
+static gboolean       tree_find_id            (GtkTreeModel     *model,
+					       GtkTreePath      *path,
+					       GtkTreeIter      *iter,
+					       hash_lookup      *hash);
 
 static YelpPagerClass *parent_class;
 
@@ -160,9 +169,8 @@ info_pager_parse (YelpPager *pager)
 {
     YelpDocInfo   *doc_info;
     gchar         *filename;
-    xmlDocPtr      doc;
-    xmlNodePtr     node;
-    GError        *error;
+    xmlDocPtr      doc = NULL;
+    GError        *error = NULL;
     YelpInfoPagerPriv *priv;
 
     g_return_val_if_fail (YELP_IS_INFO_PAGER (pager), FALSE);
@@ -177,14 +185,25 @@ info_pager_parse (YelpPager *pager)
     /* parse into a GtkTreeStore */
     priv->tree = yelp_info_parser_parse_file (filename);
 
-    /* create the XML file */
-    doc = yelp_info_parser_parse_tree (priv->tree);
+    if (!priv->tree) {
+	g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_DOC,
+		     _("The file  ‘%s’ could not be parsed. Either the file "
+		       "does not exist, or it is not a well-formed info page."),
+		     filename);
+	yelp_pager_error (pager, error);
+
+    } else {
+
+	/* create the XML file */
+	doc = yelp_info_parser_parse_tree (priv->tree);
+	
+	gtk_tree_model_foreach (GTK_TREE_MODEL (priv->tree),
+				(GtkTreeModelForeachFunc) tree_hash_id,
+				pager);
+    }
 
-    gtk_tree_model_foreach (GTK_TREE_MODEL (priv->tree),
-			    (GtkTreeModelForeachFunc) tree_hash_id,
-			    pager);
-    
     g_object_unref (pager);
+    g_free (filename);
 
     return doc;
 }
@@ -198,6 +217,13 @@ info_pager_params (YelpPager *pager)
 
     params = g_new0 (gchar *, params_max);
 
+    yelp_settings_params (&params, &params_i, &params_max);
+
+    if ((params_i + 10) >= params_max - 1) {
+	params_max += 20;
+	params = g_renew (gchar *, params, params_max);
+    }
+
     params[params_i++] = "stylesheet_path";
     params[params_i++] = g_strdup_printf ("\"file://%s\"", STYLESHEET_PATH);
 
@@ -210,10 +236,26 @@ static const gchar *
 info_pager_resolve_frag (YelpPager *pager, const gchar *frag_id)
 {
     g_return_val_if_fail (YELP_IS_INFO_PAGER (pager), NULL);
+    if (frag_id) {
+	gchar *id = g_hash_table_lookup (YELP_INFO_PAGER (pager)->priv->frags_hash,
+					 frag_id);
+	if (!id) {
+	    hash_lookup *l = g_new0 (hash_lookup, 1);
+	    
+	    l->frag_name = g_strdup (frag_id);
 
-    if (frag_id)
-	return g_hash_table_lookup (YELP_INFO_PAGER (pager)->priv->frags_hash, frag_id);
-    else
+	    gtk_tree_model_foreach (GTK_TREE_MODEL (YELP_INFO_PAGER (pager)->priv->tree),
+				    (GtkTreeModelForeachFunc) tree_find_id,
+				    l);
+	    if (l->frag_id)
+		id = g_hash_table_lookup (YELP_INFO_PAGER (pager)->priv->frags_hash,
+					  l->frag_id);
+	    g_free (l->frag_name);
+	    g_free (l->frag_id);
+	    g_free (l);
+	}
+	return id;
+    } else
 	return g_hash_table_lookup (YELP_INFO_PAGER (pager)->priv->frags_hash, "1");
 }
 
@@ -222,7 +264,7 @@ info_pager_get_sections (YelpPager *pager)
 {
     g_return_val_if_fail (YELP_IS_INFO_PAGER (pager), NULL);
 
-    return YELP_INFO_PAGER (pager)->priv->tree;
+    return GTK_TREE_MODEL (YELP_INFO_PAGER (pager)->priv->tree);
 }
 
 static gboolean
@@ -244,3 +286,21 @@ tree_hash_id (GtkTreeModel   *model,
 
     return FALSE;
 }
+
+static gboolean
+tree_find_id (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+	      hash_lookup *hash)
+{
+    gchar *title;
+    gchar *id;
+    gtk_tree_model_get (model, iter,
+			YELP_PAGER_COLUMN_ID, &id,
+			YELP_PAGER_COLUMN_TITLE, &title,
+			-1);
+    if (g_str_equal (title, hash->frag_name)) {
+	hash->frag_id = g_strdup (id);
+	return TRUE;
+    }
+    return FALSE;
+
+}
